import 'dart:async';

import 'package:grpc/grpc.dart';

import 'generated/hello.bettergrpc.dart';
import 'generated/hello.pbgrpc.dart';

const port = 1728;
const animals = ["Cat", "Dolphine", "Salmon", "Fish"];

/// Implement the logic of the server.
class HelloService extends BetterHelloServiceBase {
  @override
  FutureOr<SayHelloResponse> sayHello(User request) {
    return SayHelloResponse(message: "Hello ${request.name}!");
  }
}

void main(List<String> arguments) async {
  /// Start a server over http.
  final server = Server([
    BetterHelloService(HelloService()),
  ]);
  await server.serve(port: port);
  print('Server listening on port ${server.port}...');

  final channel = ClientChannel(
    'localhost',
    port: port,
    options: ChannelOptions(
      credentials: ChannelCredentials.insecure(),
    ),
  );
  final client = BetterHelloClient(HelloClient(channel));
  print("-- Do tests while server is running...");
  await testClient(client);
  print("-- shutdown the server..");
  await server.shutdown();
  print("-- Do tests while server is down..");
  await testClient(client);

  /// Start a offline server.
  final offlineClient = BetterHelloOfflineClient(HelloService());
  print("-- Do offline tests...");
  await testClient(offlineClient);
}

/// Test a client by doing some requests to it.
Future testClient(BetterHelloClientBase client) async {
  for (final animal in animals) {
    try {
      final response = await client.sayHello(User(name: animal));
      print(response.message);
    } catch (e) {
      if (client is BetterHelloClient) {
        print(
            "- Error, cannot execute this request on `server` seems that the server is off..");
      } else {
        print("Cannot execute this on this offline!");
        print("Unexpected, please report this issue!");
      }
    }
  }
}
